from osv import fields,osv

class report_po_month(osv.osv):
    _name="report.po.month"
    _auto=False
    _columns={
        "name":fields.char("Month",size=64,readonly=True,select=True),
        "year":fields.char("Year",size=64,readonly=True,select=True),
        "state":fields.selection([("confirmed","Confirmed"),("approved","Approved"),("done","Delivered")],"State",readonly=True,select=True),
        "price_total":fields.float("Total Price",readonly=True),
        "supplier_id":fields.many2one("res.partner","Supplier",readonly=True),
    }
    _order="name desc"

    def init(self,cr):
        cr.execute("""
create or replace view report_po_month as (
    select
        min(l.id) as id,
        to_char(po.date_order,'YYYY-MM') as name,
        to_char(po.date_order,'YYYY') as year,
        po.state,
        sum(l.product_qty*l.price_unit) as price_total,
        po.partner_id as supplier_id
    from purchase_order po
        left join purchase_order_line l on (po.id=l.order_id)
    where po.state in ('confirmed','approved','done') and l.product_id is not null
    group by to_char(po.date_order,'YYYY-MM'),to_char(po.date_order,'YYYY'),po.state,supplier_id
    order by price_total limit 20
)""")
report_po_month()

class report_po_week(osv.osv):
    _name="report.po.week"
    _auto=False
    _columns={
        "name":fields.char("Week",size=64,readonly=True,select=True),
        "state":fields.selection([("confirmed","Confirmed"),("approved","Approved"),("done","Delivered")],"State",readonly=True,select=True),
        "price_total":fields.float("Total Price",readonly=True),
        "supplier_id":fields.many2one("res.partner","Supplier",readonly=True),
    }
    _order="name desc"

    def init(self,cr):
        cr.execute("""
create or replace view report_po_week as (
    select
        min(l.id) as id,
        to_char(po.date_order,'YYYY-MM-WW') as name,
        po.state,
        sum(l.product_qty*l.price_unit) as price_total,
        po.partner_id as supplier_id
    from purchase_order po
        left join purchase_order_line l on (po.id=l.order_id)
    where po.state in ('confirmed','approved','done') and l.product_id is not null
    group by to_char(po.date_order,'YYYY-MM-WW'),po.state,supplier_id
    order by price_total limit 20
)""")
report_po_week()

class res_partner(osv.osv):
    _name="res.partner"
    _inherit="res.partner"

    def _po_month(self,cr,uid,ids,name,args,context={}):
        vals={}
        for part_id in ids:
            cr.execute("select sum(pol.product_qty*pol.price_unit) from purchase_order po left join purchase_order_line pol on pol.order_id=po.id where po.partner_id=%d and po.state in ('confirmed','approved','done') and date_order>=date_trunc('month',date 'today')",(part_id,))
            r=cr.fetchone()
            vals[part_id]=r and r[0] or 0.0
        return vals

    def _po_year(self,cr,uid,ids,name,args,context={}):
        vals={}
        for part_id in ids:
            cr.execute("select sum(pol.product_qty*pol.price_unit) from purchase_order po left join purchase_order_line pol on pol.order_id=po.id where po.partner_id=%d and po.state in ('confirmed','approved','done') and date_order>=date_trunc('year',date 'today')",(part_id,))
            r=cr.fetchone()
            vals[part_id]=r and r[0] or 0.0
        return vals

    _columns = {
        "po_month": fields.function(_po_month,method=True,type="float",string="PO This Month"),
        "po_year": fields.function(_po_year,method=True,type="float",string="PO This Year"),
    }
res_partner()
